@@ -2,7 +2,7 @@
 
   dln.c -
 
-  $Author$
+  $Author: nagachika $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
